
/* Common Open Systems Environment, Common Graphics Environment  */		
/* Extension to PEX 5.1 version 2.  File date: March 30, 1994    */		

/******************************************************************************/
/*  (c)	Copyright Hewlett-Packard Company, 1993,  Fort Collins,	Colorado      */
/*									      */
/*  (C)	COPYRIGHT International	Business Machines Corp.	1993		      */
/*									      */
/*   Copyright (c) 1993	by Sun Microsystems, Inc.			      */
/*									      */
/*			      All Rights Reserved			      */
/*									      */
/*  Permission to use, copy, modify, and distribute this software and its     */
/*  documentation for any purpose and without fee is hereby granted,	      */
/*  provided that the above copyright notices appear in	all copies and that   */
/*  both the copyright notices and this	permission notice appear in	      */
/*  supporting documentation, and that the names of Hewlett-Packard,	      */
/*  International Business Machines, or	Sun Microsystems not be	used in	      */
/*  advertising	or publicity pertaining	to distribution	of the software	      */
/*  without specific, written prior permission.				      */
/*									      */
/*  HEWLETT-PACKARD MAKES NO WARRANTY OF ANY KIND WITH REGARD TO THIS	      */
/*  SOFTWARE, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF	      */
/*  MERCHANTABILITY AND	FITNESS	FOR A PARTICULAR PURPOSE.  Hewlett-Packard    */
/*  shall not be liable	for errors contained herein or direct, indirect,      */
/*  special, incidental	or consequential damages in connection with the	      */
/*  furnishing,	performance or use of this software.			      */
/*									      */
/*  INTERNATIONAL BUSINESS MACHINES CORPORATION	PROVIDES THIS MATERIAL	      */
/*  "AS	IS" WITHOUT WARRANTY OF	ANY KIND, EITHER EXPRESSED OR IMPLIED,	      */
/*  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY  */
/*  AND	FITNESS	FOR A PARTICULAR PURPOSE.				      */
/*									      */
/*  SUN MICROSYSTEMS, INC. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS       */
/*  SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,*/
/*  IN NO EVENT SHALL SUN MICROSYSTEMS, INC. BE LIABLE FOR ANY SPECIAL,       */
/*  INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING     */
/*  FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,      */
/*  NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION      */
/*  WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.                             */
/*									      */
/******************************************************************************/


 /* =======================================================================
  * NOTE NOTE NOTE  Insert this	section
  *		    at the end of pl_oc_dec.c file
  * ====================================================================='=
  */

/***** Start of	Ext section ************/

#include "PEXExtlib.h"
#include "PEXExtlibint.h"


PEXExtOCData *
PEXExtDecodeOCs	(float_format, oc_count, length, encoded_ocs)

INPUT int		float_format;
INPUT unsigned long	oc_count;
INPUT unsigned long	length;
INPUT char		*encoded_ocs;

{
    extern void		(*(PEX_decode_oc_funcs[]))();
    extern void		(*(PEXExt_decode_oc_funcs[]))();
    PEXExtOCData	*ocDest, *ocRet;
    pexElementInfo	*elemInfo;
    char		*ocSrc;
    int			i;


    /*
     * Allocate	a buffer to hold the decoded OC	data.
     */

    ocRet = (PEXExtOCData *) PEXAllocBuf (oc_count * sizeof (PEXExtOCData));


    /*
     * Now, decode the OCs in the buffer.  Use either the standard
     * table of decode functions, the Ext table or call the default
     * Ext decode function.

     */

    ocSrc = encoded_ocs;
    ocDest = ocRet;

    for	(i = 0;	i < oc_count; i++, ocDest++)
    {
	GET_STRUCT_PTR (pexElementInfo,	ocSrc, elemInfo);
	ocDest->oc_type	= elemInfo->elementType;
        if ( (short) elemInfo->elementType  >= 0         &&
             (short) elemInfo->elementType  <= PEXOCNoop   )
        {
	   (*PEX_decode_oc_funcs[elemInfo->elementType])
	       (float_format, &ocSrc, ocDest);
        }

        else if ( (short) elemInfo->elementType >= PEXExtOCMin &&
                  (short) elemInfo->elementType <= PEXExtOCMax )
	{
	   (*PEXExt_decode_oc_funcs
	       [elemInfo->elementType -	(unsigned short) PEXExtOCMin]) 
	       (float_format, &ocSrc, ocDest);
	}

        else
        {
            _PEXExtDecodeVendorExtOC
                (&ocSrc, ocDest);
        }
    }

#ifdef DEBUG
    if (ocSrc -	encoded_ocs != length)
    {
	printf ("PEXlib	WARNING	: Internal error in PEXExtDecodeOCs :\n");
	printf ("Number	of bytes parsed	not equal to size of input buffer.\n");
    }
#endif

    return (ocRet);
}


void _PEXExtDecodeEnumType (fpFormat, ocSrc, ocDest)

int		fpFormat;
char		**ocSrc;
PEXExtOCData	*ocDest;

{
    pexExtTMPerspectiveCorrection *oc;

    GET_STRUCT_PTR (pexExtTMPerspectiveCorrection, *ocSrc, oc);
    *ocSrc += SIZEOF (pexExtTMPerspectiveCorrection);

    ocDest->data.ExtSetTMPerspectiveCorrection.method =	oc->correction;
}


void _PEXExtDecodeTMResourceHints (fpFormat, ocSrc, ocDest)

int		fpFormat;
char		**ocSrc;
PEXExtOCData	*ocDest;

{
    pexExtTMResourceHints *oc;

    GET_STRUCT_PTR (pexExtTMResourceHints, *ocSrc, oc);
    *ocSrc += SIZEOF (pexExtTMResourceHints);
    
    ocDest->data.ExtSetTMResourceHints.optimization = oc->optimization;
    ocDest->data.ExtSetTMResourceHints.count = oc->numIndex;

    ocDest->data.ExtSetTMResourceHints.priorities = (PEXTableIndex *)
	PEXAllocBuf (oc->numIndex * sizeof (PEXTableIndex));

    EXTRACT_LISTOF_CARD16 (oc->numIndex, *ocSrc,
	ocDest->data.ExtSetTMResourceHints.priorities);

    if (oc->numIndex & 1)
	*ocSrc += 2;
}


void _PEXExtDecodeActiveTextures (fpFormat, ocSrc, ocDest)

int		fpFormat;
char		**ocSrc;
PEXExtOCData	*ocDest;

{
    pexExtActiveTextures *oc;

    GET_STRUCT_PTR (pexExtActiveTextures, *ocSrc, oc);
    *ocSrc += SIZEOF (pexExtActiveTextures);
    
    ocDest->data.ExtSetActiveTextures.count = oc->numIndex;

    ocDest->data.ExtSetActiveTextures.textures = (PEXTableIndex	*)
	PEXAllocBuf (oc->numIndex * sizeof (PEXTableIndex));

    EXTRACT_LISTOF_CARD16 (oc->numIndex, *ocSrc,
	ocDest->data.ExtSetActiveTextures.textures);

    if (oc->numIndex & 1)
	*ocSrc += 2;
}


void _PEXExtDecodeFillAreaSetWithData (fpFormat, ocSrc,	ocDest)

int		fpFormat;
char		**ocSrc;
PEXExtOCData	*ocDest;

{
    pexExtFillAreaSetWithData	*oc;
    PEXExtListOfVertex		*pList;
    int				vertexSize, i;
    int				fpConvert = (fpFormat != NATIVE_FP_FORMAT);

    GET_STRUCT_PTR (pexExtFillAreaSetWithData, *ocSrc, oc);
    *ocSrc += SIZEOF (pexExtFillAreaSetWithData);

    ocDest->data.ExtFillAreaSetWithData.shape_hint = oc->shape;
    ocDest->data.ExtFillAreaSetWithData.vertex_fp_data_size = oc->numVertexFPData;
    ocDest->data.ExtFillAreaSetWithData.ignore_edges = oc->ignoreEdges;
    ocDest->data.ExtFillAreaSetWithData.contour_hint = oc->contourHint;
    ocDest->data.ExtFillAreaSetWithData.facet_attributes = oc->facetAttribs;
    ocDest->data.ExtFillAreaSetWithData.vertex_attributes = oc->vertexAttribs;
    ocDest->data.ExtFillAreaSetWithData.color_type = oc->colorType;

    if (oc->facetAttribs)
    {
	EXTRACT_FACET (*ocSrc, oc->colorType, oc->facetAttribs,
	    ocDest->data.ExtFillAreaSetWithData.facet_data, fpConvert, 
            fpFormat);
    }

    ocDest->data.ExtFillAreaSetWithData.count = oc->numLists;
    ocDest->data.ExtFillAreaSetWithData.vertex_lists = pList = 
        (PEXExtListOfVertex *)
	PEXAllocBuf (oc->numLists * sizeof (PEXExtListOfVertex));
    
    vertexSize = GetClientVertexSize (oc->colorType, oc->vertexAttribs);
    if (oc->vertexAttribs & PEXGAEdges)
	vertexSize += sizeof (unsigned int);
  
    if (oc->vertexAttribs & PEXExtGAData)
	vertexSize += (oc->numVertexFPData * sizeof (float));

    for	(i = 0;	i < oc->numLists; i++, pList++)
    {
	EXTRACT_CARD32 (*ocSrc,	pList->count);

	pList->vertices.no_data	= (PEXCoord *) PEXAllocBuf (
	    pList->count * vertexSize);

	EXTRACT_LISTOF_EXT_VERTEX (pList->count, *ocSrc, vertexSize,
	    oc->colorType, oc->numVertexFPData, oc->vertexAttribs,
	    pList->vertices, fpConvert,	fpFormat);
    }
}


void _PEXExtDecodeTriangleStrip	(fpFormat, ocSrc, ocDest)

int		fpFormat;
char		**ocSrc;
PEXExtOCData	*ocDest;

{
    pexExtTriangleStrip	*oc;
    int			facetSize;
    int			vertexSize;
    int			fpConvert = (fpFormat != NATIVE_FP_FORMAT);

    GET_STRUCT_PTR (pexExtTriangleStrip, *ocSrc, oc);
    *ocSrc += SIZEOF (pexExtTriangleStrip);

    ocDest->data.ExtTriangleStrip.facet_attributes = oc->facetAttribs;
    ocDest->data.ExtTriangleStrip.vertex_attributes = oc->vertexAttribs;
    ocDest->data.ExtTriangleStrip.color_type = oc->colorType;
    ocDest->data.ExtTriangleStrip.vertex_fp_data_size = oc->numVertexFPData;
    ocDest->data.ExtTriangleStrip.count = oc->numVertices;

    if (oc->facetAttribs)
    {
	facetSize = GetClientFacetSize (oc->colorType, oc->facetAttribs);

	ocDest->data.ExtTriangleStrip.facet_data.index = (PEXColorIndexed *)
	    PEXAllocBuf	((oc->numVertices - 2) * facetSize);

	EXTRACT_LISTOF_FACET ((oc->numVertices - 2), *ocSrc, facetSize,
	    oc->colorType, oc->facetAttribs,
	    ocDest->data.ExtTriangleStrip.facet_data, fpConvert, fpFormat);
    }
    else
	ocDest->data.ExtTriangleStrip.facet_data.index = NULL;

    vertexSize = GetClientVertexSize (oc->colorType, oc->vertexAttribs);

    if (oc->vertexAttribs & PEXExtGAData)
	vertexSize += (oc->numVertexFPData * sizeof (float));

    ocDest->data.ExtTriangleStrip.vertices.no_data	=
	(PEXCoord *) PEXAllocBuf (oc->numVertices * vertexSize);

    EXTRACT_LISTOF_EXT_VERTEX (oc->numVertices, *ocSrc, vertexSize,
	oc->colorType, oc->numVertexFPData, oc->vertexAttribs,
	ocDest->data.ExtTriangleStrip.vertices, fpConvert, fpFormat);
}


void _PEXExtDecodeQuadMesh (fpFormat, ocSrc, ocDest)

int		fpFormat;
char		**ocSrc;
PEXExtOCData	*ocDest;

{
    pexExtQuadrilateralMesh	*oc;
    int				count;
    int				facetSize;
    int				vertexSize;
    int				fpConvert = (fpFormat != NATIVE_FP_FORMAT);

    GET_STRUCT_PTR (pexExtQuadrilateralMesh, *ocSrc, oc);
    *ocSrc += SIZEOF (pexExtQuadrilateralMesh);

    ocDest->data.ExtQuadrilateralMesh.shape_hint = oc->shape;
    ocDest->data.ExtQuadrilateralMesh.facet_attributes = oc->facetAttribs;
    ocDest->data.ExtQuadrilateralMesh.vertex_attributes = oc->vertexAttribs;
    ocDest->data.ExtQuadrilateralMesh.color_type = oc->colorType;
    ocDest->data.ExtQuadrilateralMesh.vertex_fp_data_size = oc->numVertexFPData;
    ocDest->data.ExtQuadrilateralMesh.col_count = oc->mPts;
    ocDest->data.ExtQuadrilateralMesh.row_count = oc->nPts;

    if (oc->facetAttribs)
    {
	facetSize = GetClientFacetSize (oc->colorType, oc->facetAttribs);

	count =	(oc->mPts - 1) * (oc->nPts - 1);
	ocDest->data.ExtQuadrilateralMesh.facet_data.index	=
	    (PEXColorIndexed *)	PEXAllocBuf (count * facetSize);

	EXTRACT_LISTOF_FACET (count, *ocSrc, facetSize,
	    oc->colorType, oc->facetAttribs,
	    ocDest->data.ExtQuadrilateralMesh.facet_data, fpConvert, fpFormat);
    }
    else
	ocDest->data.ExtQuadrilateralMesh.facet_data.index = NULL;

    vertexSize = GetClientVertexSize (oc->colorType, oc->vertexAttribs);

    if (oc->vertexAttribs & PEXExtGAData)
	vertexSize += (oc->numVertexFPData * sizeof (float));

    count = oc->mPts * oc->nPts;
    ocDest->data.ExtQuadrilateralMesh.vertices.no_data =
	(PEXCoord *) PEXAllocBuf (count	* vertexSize);

    EXTRACT_LISTOF_EXT_VERTEX (count, *ocSrc, vertexSize,
	oc->colorType, oc->numVertexFPData, oc->vertexAttribs,
	ocDest->data.ExtQuadrilateralMesh.vertices, fpConvert, fpFormat);
}


void _PEXExtDecodeSOFA (fpFormat, ocSrc, ocDest)

int		fpFormat;
char		**ocSrc;
PEXExtOCData	*ocDest;

{
    pexExtSetOfFillAreaSets	*oc;
    PEXConnectivityData		*pCon;
    PEXListOfUShort		*pList;
    int				cbytes;
    int				facetSize;
    int				vertexSize;
    int				i, j;
    int				fpConvert = (fpFormat != NATIVE_FP_FORMAT);

    GET_STRUCT_PTR (pexExtSetOfFillAreaSets, *ocSrc, oc);
    *ocSrc += SIZEOF (pexExtSetOfFillAreaSets);

    ocDest->data.ExtSetOfFillAreaSets.shape_hint = oc->shape;
    ocDest->data.ExtSetOfFillAreaSets.facet_attributes = oc->FAS_Attributes;
    ocDest->data.ExtSetOfFillAreaSets.vertex_attributes = oc->vertexAttributes;
    ocDest->data.ExtSetOfFillAreaSets.edge_attributes =
	(oc->edgeAttributes == PEXOn) ?	PEXGAEdges : 0;
    ocDest->data.ExtSetOfFillAreaSets.contour_hint = oc->contourHint;
    ocDest->data.ExtSetOfFillAreaSets.contours_all_one = oc->contourCountsFlag;
    ocDest->data.ExtSetOfFillAreaSets.color_type = oc->colorType;
    ocDest->data.ExtSetOfFillAreaSets.vertex_fp_data_size = oc->numVertexFPData;
    ocDest->data.ExtSetOfFillAreaSets.set_count = oc->numFAS;
    ocDest->data.ExtSetOfFillAreaSets.vertex_count = oc->numVertices;
    ocDest->data.ExtSetOfFillAreaSets.index_count = oc->numEdges;

    if (oc->FAS_Attributes)
    {
	facetSize = GetClientFacetSize (oc->colorType, oc->FAS_Attributes);

	ocDest->data.ExtSetOfFillAreaSets.facet_data.index =
	    (PEXColorIndexed *)	PEXAllocBuf (oc->numFAS	* facetSize);

	EXTRACT_LISTOF_FACET (oc->numFAS, *ocSrc, facetSize,
	    oc->colorType, oc->FAS_Attributes,
	    ocDest->data.ExtSetOfFillAreaSets.facet_data, fpConvert, fpFormat);
    }
    else
	ocDest->data.ExtSetOfFillAreaSets.facet_data.index = NULL;

    vertexSize = GetClientVertexSize (oc->colorType, oc->vertexAttributes);

    if (oc->vertexAttributes & PEXExtGAData)
	vertexSize += (oc->numVertexFPData * sizeof (float));

    ocDest->data.ExtSetOfFillAreaSets.vertices.no_data =
	(PEXCoord *) PEXAllocBuf (oc->numVertices * vertexSize);

    EXTRACT_LISTOF_EXT_VERTEX (oc->numVertices, *ocSrc, vertexSize,
	oc->colorType, oc->numVertexFPData, oc->vertexAttributes,
	ocDest->data.ExtSetOfFillAreaSets.vertices, fpConvert, fpFormat);

    if (oc->edgeAttributes)
    {
	unsigned int size = oc->numEdges * sizeof (CARD8);
	ocDest->data.ExtSetOfFillAreaSets.edge_flags =
	    (PEXSwitch *) PEXAllocBuf (size);
	COPY_AREA (*ocSrc, ocDest->data.ExtSetOfFillAreaSets.edge_flags, size);
	*ocSrc += PADDED_BYTES (size);
    }
    else
	ocDest->data.ExtSetOfFillAreaSets.edge_flags = NULL;
	
    ocDest->data.ExtSetOfFillAreaSets.connectivity = pCon =
	(PEXConnectivityData *)	PEXAllocBuf (oc->numFAS	*
	sizeof (PEXConnectivityData));

    for	(i = 0;	i < (int) oc->numFAS; i++, pCon++)
    {
	EXTRACT_CARD16 (*ocSrc,	pCon->count);

	pCon->lists = pList = (PEXListOfUShort *)
	    PEXAllocBuf	(pCon->count * sizeof (PEXListOfUShort));

	for (j = 0; j <	(int) pCon->count; j++,	pList++)
	{
	    EXTRACT_CARD16 (*ocSrc, pList->count);

	    pList->shorts = (unsigned short *) PEXAllocBuf (
		pList->count * sizeof (unsigned	short));

	    EXTRACT_LISTOF_CARD16 (pList->count, *ocSrc, pList->shorts);
	}
    }
	
    cbytes = SIZEOF (CARD16) * (oc->numFAS + oc->numContours + oc->numEdges);
    *ocSrc += PAD (cbytes);
}


void _PEXExtDecodePrimitiveAA (fpFormat, ocSrc,	ocDest)

int		fpFormat;
char		**ocSrc;
PEXExtOCData	*ocDest;

{
    pexExtPrimitiveAA *oc;

    GET_STRUCT_PTR (pexExtPrimitiveAA, *ocSrc, oc);
    *ocSrc += SIZEOF (pexExtPrimitiveAA);

    ocDest->data.ExtSetPrimitiveAA.mode	= oc->mode;
    ocDest->data.ExtSetPrimitiveAA.blend_op = oc->blend_op;
}


void _PEXExtDecodeEllipse (fpFormat, ocSrc, ocDest)

int		fpFormat;
char		**ocSrc;
PEXExtOCData	*ocDest;

{
    pexExtEllipse	*oc;
    int			fpConvert = (fpFormat != NATIVE_FP_FORMAT);


    GET_STRUCT_PTR (pexExtEllipse, *ocSrc, oc);
    *ocSrc += SIZEOF (pexExtEllipse);

    if (fpConvert)
    {
	FP_CONVERT_NTOH	(oc->center_x,
	    ocDest->data.ExtEllipse.center.x, fpFormat);
	FP_CONVERT_NTOH	(oc->center_y,
	    ocDest->data.ExtEllipse.center.y, fpFormat);
	FP_CONVERT_NTOH	(oc->center_z,
	    ocDest->data.ExtEllipse.center.z, fpFormat);
	FP_CONVERT_NTOH	(oc->major_x,
	    ocDest->data.ExtEllipse.major.x, fpFormat);
	FP_CONVERT_NTOH	(oc->major_y,
	    ocDest->data.ExtEllipse.major.y, fpFormat);
	FP_CONVERT_NTOH	(oc->major_z,
	    ocDest->data.ExtEllipse.major.z, fpFormat);
	FP_CONVERT_NTOH	(oc->minor_x,
	    ocDest->data.ExtEllipse.minor.x, fpFormat);
	FP_CONVERT_NTOH	(oc->minor_y,
	    ocDest->data.ExtEllipse.minor.y, fpFormat);
	FP_CONVERT_NTOH	(oc->minor_z,
	    ocDest->data.ExtEllipse.minor.z, fpFormat);
    }
    else
    {
	ocDest->data.ExtEllipse.center.x = oc->center_x;
	ocDest->data.ExtEllipse.center.y = oc->center_y;
	ocDest->data.ExtEllipse.center.z = oc->center_z;
	ocDest->data.ExtEllipse.major.x	= oc->major_x;
	ocDest->data.ExtEllipse.major.y	= oc->major_y;
	ocDest->data.ExtEllipse.major.z	= oc->major_z;
	ocDest->data.ExtEllipse.minor.x	= oc->minor_x;
	ocDest->data.ExtEllipse.minor.y	= oc->minor_y;
	ocDest->data.ExtEllipse.minor.z	= oc->minor_z;
    }
}


void _PEXExtDecodeEllipse2D (fpFormat, ocSrc, ocDest)

int		fpFormat;
char		**ocSrc;
PEXExtOCData	*ocDest;

{
    pexExtEllipse2D	*oc;
    int			fpConvert = (fpFormat != NATIVE_FP_FORMAT);


    GET_STRUCT_PTR (pexExtEllipse2D, *ocSrc, oc);
    *ocSrc += SIZEOF (pexExtEllipse2D);

    if (fpConvert)
    {
	FP_CONVERT_NTOH	(oc->center_x,
	    ocDest->data.ExtEllipse2D.center.x,	fpFormat);
	FP_CONVERT_NTOH	(oc->center_y,
	    ocDest->data.ExtEllipse2D.center.y,	fpFormat);
	FP_CONVERT_NTOH	(oc->major_x,
	    ocDest->data.ExtEllipse2D.major.x, fpFormat);
	FP_CONVERT_NTOH	(oc->major_y,
	    ocDest->data.ExtEllipse2D.major.y, fpFormat);
	FP_CONVERT_NTOH	(oc->minor_x,
	    ocDest->data.ExtEllipse2D.minor.x, fpFormat);
	FP_CONVERT_NTOH	(oc->minor_y,
	    ocDest->data.ExtEllipse2D.minor.y, fpFormat);
    }
    else
    {
	ocDest->data.ExtEllipse2D.center.x = oc->center_x;
	ocDest->data.ExtEllipse2D.center.y = oc->center_y;
	ocDest->data.ExtEllipse2D.major.x = oc->major_x;
	ocDest->data.ExtEllipse2D.major.y = oc->major_y;
	ocDest->data.ExtEllipse2D.minor.x = oc->minor_x;
	ocDest->data.ExtEllipse2D.minor.y = oc->minor_y;
    }
}


void _PEXExtDecodeCircle2D (fpFormat, ocSrc, ocDest)

int		fpFormat;
char		**ocSrc;
PEXExtOCData	*ocDest;

{
    pexExtCircle2D	*oc;
    int			fpConvert = (fpFormat != NATIVE_FP_FORMAT);


    GET_STRUCT_PTR (pexExtCircle2D, *ocSrc, oc);
    *ocSrc += SIZEOF (pexExtCircle2D);

    if (fpConvert)
    {
	FP_CONVERT_NTOH	(oc->center_x,
	    ocDest->data.ExtCircle2D.center.x, fpFormat);
	FP_CONVERT_NTOH	(oc->center_y,
	    ocDest->data.ExtCircle2D.center.y, fpFormat);
	FP_CONVERT_NTOH	(oc->radius,
	    ocDest->data.ExtCircle2D.radius, fpFormat);
    }
    else
    {
	ocDest->data.ExtCircle2D.center.x = oc->center_x;
	ocDest->data.ExtCircle2D.center.y = oc->center_y;
	ocDest->data.ExtCircle2D.radius	= oc->radius;
    }
}


void _PEXExtDecodeEllipticalArc	(fpFormat, ocSrc, ocDest)

int		fpFormat;
char		**ocSrc;
PEXExtOCData	*ocDest;

{
    pexExtEllipticalArc	*oc;
    int			fpConvert = (fpFormat != NATIVE_FP_FORMAT);


    GET_STRUCT_PTR (pexExtEllipticalArc, *ocSrc, oc);
    *ocSrc += SIZEOF (pexExtEllipticalArc);

    if (fpConvert)
    {
	FP_CONVERT_NTOH	(oc->center_x,
	    ocDest->data.ExtEllipticalArc.center.x, fpFormat);
	FP_CONVERT_NTOH	(oc->center_y,
	    ocDest->data.ExtEllipticalArc.center.y, fpFormat);
	FP_CONVERT_NTOH	(oc->center_z,
	    ocDest->data.ExtEllipticalArc.center.z, fpFormat);
	FP_CONVERT_NTOH	(oc->major_x,
	    ocDest->data.ExtEllipticalArc.major.x, fpFormat);
	FP_CONVERT_NTOH	(oc->major_y,
	    ocDest->data.ExtEllipticalArc.major.y, fpFormat);
	FP_CONVERT_NTOH	(oc->major_z,
	    ocDest->data.ExtEllipticalArc.major.z, fpFormat);
	FP_CONVERT_NTOH	(oc->minor_x,
	    ocDest->data.ExtEllipticalArc.minor.x, fpFormat);
	FP_CONVERT_NTOH	(oc->minor_y,
	    ocDest->data.ExtEllipticalArc.minor.y, fpFormat);
	FP_CONVERT_NTOH	(oc->minor_z,
	    ocDest->data.ExtEllipticalArc.minor.z, fpFormat);
	FP_CONVERT_NTOH	(oc->start_angle,  
	    ocDest->data.ExtEllipticalArc.start, fpFormat);
	FP_CONVERT_NTOH	(oc->stop_angle,  
	    ocDest->data.ExtEllipticalArc.stop,	fpFormat);
    }
    else
    {
	ocDest->data.ExtEllipticalArc.center.x = oc->center_x;
	ocDest->data.ExtEllipticalArc.center.y = oc->center_y;
	ocDest->data.ExtEllipticalArc.center.z = oc->center_z;
	ocDest->data.ExtEllipticalArc.major.x =	oc->major_x;
	ocDest->data.ExtEllipticalArc.major.y =	oc->major_y;
	ocDest->data.ExtEllipticalArc.major.z =	oc->major_z;
	ocDest->data.ExtEllipticalArc.minor.x =	oc->minor_x;
	ocDest->data.ExtEllipticalArc.minor.y =	oc->minor_y;
	ocDest->data.ExtEllipticalArc.minor.z =	oc->minor_z;
	ocDest->data.ExtEllipticalArc.start = oc->start_angle;
	ocDest->data.ExtEllipticalArc.stop = oc->stop_angle;
    }
}


void _PEXExtDecodeEllipticalArc2D (fpFormat, ocSrc, ocDest)

int		fpFormat;
char		**ocSrc;
PEXExtOCData	*ocDest;

{
    pexExtEllipticalArc2D	*oc;
    int			fpConvert = (fpFormat != NATIVE_FP_FORMAT);


    GET_STRUCT_PTR (pexExtEllipticalArc2D, *ocSrc, oc);
    *ocSrc += SIZEOF (pexExtEllipticalArc2D);

    if (fpConvert)
    {
	FP_CONVERT_NTOH	(oc->center_x,
	    ocDest->data.ExtEllipticalArc2D.center.x, fpFormat);
	FP_CONVERT_NTOH	(oc->center_y,
	    ocDest->data.ExtEllipticalArc2D.center.y, fpFormat);
	FP_CONVERT_NTOH	(oc->major_x,
	    ocDest->data.ExtEllipticalArc2D.major.x, fpFormat);
	FP_CONVERT_NTOH	(oc->major_y,
	    ocDest->data.ExtEllipticalArc2D.major.y, fpFormat);
	FP_CONVERT_NTOH	(oc->minor_x,
	    ocDest->data.ExtEllipticalArc2D.minor.x, fpFormat);
	FP_CONVERT_NTOH	(oc->minor_y,
	    ocDest->data.ExtEllipticalArc2D.minor.y, fpFormat);
	FP_CONVERT_NTOH	(oc->start_angle,
	    ocDest->data.ExtEllipticalArc2D.start, fpFormat);
	FP_CONVERT_NTOH	(oc->stop_angle,
	    ocDest->data.ExtEllipticalArc2D.stop, fpFormat);
    }
    else
    {
	ocDest->data.ExtEllipticalArc2D.center.x = oc->center_x;
	ocDest->data.ExtEllipticalArc2D.center.y = oc->center_y;
	ocDest->data.ExtEllipticalArc2D.major.x	= oc->major_x;
	ocDest->data.ExtEllipticalArc2D.major.y	= oc->major_y;
	ocDest->data.ExtEllipticalArc2D.minor.x	= oc->minor_x;
	ocDest->data.ExtEllipticalArc2D.minor.y	= oc->minor_y;
	ocDest->data.ExtEllipticalArc2D.start =	oc->start_angle;
	ocDest->data.ExtEllipticalArc2D.stop = oc->stop_angle;
    }
}


void _PEXExtDecodeCircularArc2D	(fpFormat, ocSrc, ocDest)
int		fpFormat;
char		**ocSrc;
PEXExtOCData	*ocDest;

{
    pexExtCircularArc2D	*oc;
    int			fpConvert = (fpFormat != NATIVE_FP_FORMAT);


    GET_STRUCT_PTR (pexExtCircularArc2D, *ocSrc, oc);
    *ocSrc += SIZEOF (pexExtCircularArc2D);

    if (fpConvert)
    {
	FP_CONVERT_NTOH	(oc->center_x,
	    ocDest->data.ExtCircularArc2D.center.x, fpFormat);
	FP_CONVERT_NTOH	(oc->center_y,
	    ocDest->data.ExtCircularArc2D.center.y, fpFormat);
	FP_CONVERT_NTOH	(oc->radius,
	    ocDest->data.ExtCircularArc2D.radius, fpFormat);
	FP_CONVERT_NTOH	(oc->start_angle,
	    ocDest->data.ExtCircularArc2D.start, fpFormat);
	FP_CONVERT_NTOH	(oc->stop_angle,
	    ocDest->data.ExtCircularArc2D.stop,	fpFormat);
    }
    else
    {
	ocDest->data.ExtCircularArc2D.center.x = oc->center_x;
	ocDest->data.ExtCircularArc2D.center.y = oc->center_y;
	ocDest->data.ExtCircularArc2D.radius = oc->radius;
	ocDest->data.ExtCircularArc2D.start = oc->start_angle;
	ocDest->data.ExtCircularArc2D.stop = oc->stop_angle;
    }
}


void _PEXExtDecodeVendorExtOC (ocSrc, ocDest)

char		**ocSrc;
PEXExtOCData	*ocDest;

{
    pexExtVendorExtOC		*oc;

    GET_STRUCT_PTR (pexExtVendorExtOC, *ocSrc, oc);
    *ocSrc += SIZEOF (pexExtVendorExtOC);

    ocDest->data.ExtVendorExtOC.length = oc->numBytes;

    ocDest->data.ExtVendorExtOC.data = (char *) PEXAllocBuf (oc->numBytes);

    
    COPY_AREA (*ocSrc, ocDest->data.ExtVendorExtOC.data, oc->numBytes);
    *ocSrc += PADDED_BYTES (oc->numBytes);
	
}
/***** End of Ext section ***********/
